'use strict';

const fs = require('fs');
const path = require('path');
const assert = require('assert');

const Benchpress = require('../lib/benchpress');
const { equalsIgnoreWhitespace } = require('./lib/utils');
const mainData = require('./data.json');

const source = fs.readFileSync(path.join(__dirname, 'templates/source/loop-inside-if-else.tpl')).toString();
const expected = fs.readFileSync(path.join(__dirname, 'templates/expected/loop-inside-if-else.html')).toString();

describe('compileRender', () => {
  before(() => {
    Benchpress.flush();
  });

  it('should work', () =>
    Benchpress.compileRender(source, mainData)
      .then(output => equalsIgnoreWhitespace(output, expected))
  );

  it('should work with block', () =>
    Benchpress.compileRender(source, mainData, 'rooms')
      .then(output => equalsIgnoreWhitespace(output, expected))
  );
});

describe('compileParse', () => {
  before(() => {
    Benchpress.flush();
  });

  it('should work', (done) => {
    Benchpress.compileParse(source, mainData, (err, output) => {
      assert.ifError(err);

      equalsIgnoreWhitespace(output, expected);
      done();
    });
  });

  it('should work with block', (done) => {
    Benchpress.compileParse(source, 'rooms', mainData, (err, output) => {
      assert.ifError(err);

      equalsIgnoreWhitespace(output, expected);
      done();
    });
  });
});
